from rasa.engine.graph import GraphComponent, ExecutionContext
from rasa.engine.recipes.default_recipe import DefaultV1Recipe
from rasa.engine.storage.resource import Resource
from rasa.engine.storage.storage import ModelStorage
from rasa.shared.nlu.training_data.message import Message
from rasa.shared.nlu.training_data.training_data import TrainingData
import unicodedata
from typing import Any, Dict, List, Optional

@DefaultV1Recipe.register(DefaultV1Recipe.ComponentType.MESSAGE_TOKENIZER, is_trainable=False)
class AccentNormalizer(GraphComponent):
    def __init__(self, config: Dict[str, Any]) -> None:
        self._config = config

    @staticmethod
    def get_default_config() -> Dict[str, Any]:
        return {}

    def normalize(self, text: str) -> str:
        return ''.join(
            c for c in unicodedata.normalize('NFD', text)
            if unicodedata.category(c) != 'Mn'
        )

    def process(self, messages: List[Message]) -> List[Message]:
        for message in messages:
            text = message.get("text", "")
            normalized_text = self.normalize(text)
            message.set("text", normalized_text)
        return messages

    def process_training_data(self, training_data: TrainingData) -> TrainingData:
        for example in training_data.training_examples:
            text = example.get("text", "")
            normalized_text = self.normalize(text)
            example.set("text", normalized_text)
        return training_data

    def train(self, training_data: TrainingData) -> None:
        for example in training_data.training_examples:
            normalized_text = self.normalize(example.text)
            example.text = normalized_text

    @classmethod
    def create(
        cls,
        config: Dict[str, Any],
        model_storage: ModelStorage,
        resource: Resource,
        execution_context: ExecutionContext,
    ) -> GraphComponent:
        return cls(config)

