<?php

namespace Aunnait\Rasalicante;
use Symfony\Component\Yaml\Yaml;

class RasaTraining {

    public function getDefaultDomainYaml() {
        $domain = [
            "intents" => [],
            "entities" => [],
            "slots" => [],
            "responses" => [],
            "actions" => ["action_limpiar_formulario", "action_terminar_formulario"],
            "forms" => [],
            "session_config" => ["session_expiration_time" => 60, "carry_over_slots_to_new_session" => true]
        ];
        

        return $domain;
    }

    public function createDomainYaml($bot, $language) {
        // Crear domain.yml
        $domain = $this->getDefaultDomainYaml();

        // obtener las entidades extrayendolas los valores del campo "name" de la tabla "concepts" cuyo chatbot_id sea el mismo que el bot
        $concepts = \DB::table('concepts')->where('chatbot_id', $bot)->get();
        foreach ($concepts as $concept) {
            $domain["entities"][] = strtolower(str_replace(" ", "_", $concept->name));
        }

        // obtener las intenciones extrayendolas los valores del campo "name" de la tabla "intentions" cuyo chatbot_id sea el mismo que el bot
        $intents = \DB::table('intentions')->where('chatbot_id', $bot)->whereNull("deleted_at")->get();
        foreach ($intents as $intent) {
            // El nombre almacenado es en minúsculas y con los espacios sustituidos por "_"
            $domain["intents"][] = strtolower(str_replace(" ", "_", $intent->name));
        }


        foreach ($domain["entities"] as $concept) {
            $domain["slots"][$concept] = ["type" => "text", "mappings" => [["type" => "from_entity", "entity" =>  $concept]]];
        }


        $forms = \DB::table('intentions')->where('chatbot_id', $bot)->whereNull("deleted_at")->get();
        foreach ($forms as $form) {
            $form_concepts = \DB::table('intentions_concepts')->where('intention_id', $form->id)->get();
            $form_concepts_array = [];
            foreach ($form_concepts as $form_concept) {
                $form_concepts_array[] = strtolower(str_replace(" ", "_", \DB::table('concepts')->where('id', $form_concept->concept_id)->first()->name));
            }
            // if $form_concepts_array is empty, don't add it to the forms
            if (count($form_concepts_array) > 0)
                $domain["forms"][strtolower(str_replace(" ","_",$form->name))."_form"] = ["required_slots" => $form_concepts_array];
        }

        // Starting from the intents, get the intentions_conepts elements where its intention_id is the same as the intent
        // Then get the concepts table elements where id is the concept_id from intentions_conecpts
        // Finally get the name column from concept_languages table elements which concept_id field is the id from the concepts table, filtering by the language column
        foreach ($forms as $form) {
            $form_concepts = \DB::table('intentions_concepts')->where('intention_id', $form->id)->get();
            $form_concepts_array = [];
            foreach ($form_concepts as $form_concept) {
                $concept_name = strtolower(str_replace(" ", "_", \DB::table('concepts')->where('id', $form_concept->concept_id)->first()->name));
                $concept_question = \DB::table('concept_languages')->where('concept_id', $form_concept->concept_id)->where('language', $language)->first()->question;

                $domain["responses"]["utter_ask_".$concept_name][] = ["text" => $concept_question];
            }
        }

        // Got to get the answers from the answers table
        $answers = \DB::table('answers_languages')
            ->join('answers', 'answers_languages.answers_id', '=', 'answers.id')
            ->join('intentions', 'answers.intentions_id', '=', 'intentions.id')
            ->leftJoin('intentions_concepts', 'intentions.id', '=', 'intentions_concepts.intention_id')
            ->whereNull("intentions.deleted_at")
            ->where('intentions.chatbot_id', $bot)
            ->where('answers_languages.language', $language)
            ->whereNull('intentions_concepts.intention_id')
            ->select('answers_languages.answers', 'intentions.name')
            ->get();
        foreach ($answers as $answer) {
            $answerText = $answer->answers;
            $intentionName = str_replace(' ', '_', strtolower($answer->name));
            $domain["responses"]["utter_".$intentionName][] = ["text" => $answerText];
        }
        // Si no existe $domain["responses"]["utter_no_le_he_entendido"] se añade
        if (!isset($domain["responses"]["utter_no_le_he_entendido"])) {
            $domain["responses"]["utter_no_le_he_entendido"][] = ["text" => "No le he entendido por defecto, ¿puede repetirlo?"];
        }

        // Si no hay un $domain["responses"]["utter_cancelar"] se añade
        if (!isset($domain["responses"]["utter_cancelar"])) {
            $domain["responses"]["utter_cancelar"][] = ["text" => "Cancelamos la pregunta ¿Que trámite quiere hacer?"];
        }
        
        if (empty($domain['entities'])) {
            unset($domain['entities']);
        }
        return Yaml::dump($domain, 6, 2);

    }

    public function getDefaultNluYaml() {
        $nlu = [
            "nlu" => []
        ];

        return $nlu;
    }
    public function createNluYaml($bot,$language) {

        $nlu = $this->getDefaultNluYaml();
        
        $lists = \DB::table('lists')
            ->where('chatbot_id', $bot)
            ->get();

        foreach ($lists as $list) {
            // Consultar los términos de la lista en el idioma específico
            $terminos_genericos = \DB::table('list_terms')
                ->where('list_id', $list->id)
                ->pluck('id');

            $terms = \DB::table('terms_language')
                ->whereIn('list_term_id', $terminos_genericos)
                ->where('language', $language)
                ->pluck('lang_term');

            if ($terms->isNotEmpty()) {
                $nlu['nlu'][] = [
                    'lookup' => $list->name,
                    'examples' => "- ". implode("\n- ", $terms->toArray())
                    //'examples' => $terms->toArray()//"| \n     - " . implode("\n     - ", $terms->toArray())
                ];
            }
        }
        // Consultar las listas asociadas al chatbot ID especificado
        $listIds = \DB::table('lists')
            ->where('chatbot_id', $bot)
            ->pluck('id');

        // Consultar los sinónimos que están ligados a los términos de las listas del bot
        $synonyms = \DB::table('synonyms')
            ->join('synonym_terms', 'synonyms.id', '=', 'synonym_terms.synonym_id')
            ->join('list_terms', 'synonym_terms.term_id', '=', 'list_terms.id')
            ->where('synonym_terms.language', $language)
            ->whereIn('list_terms.list_id', $listIds)
            ->select('synonyms.id', 'synonyms.synonym', 'synonym_terms.term')
            ->get();

        // Agrupar los sinónimos y sus términos correspondientes
        $synonymData = [];
        foreach ($synonyms as $synonym) {
            $synonymData[$synonym->term][] = $synonym->synonym;
        }

        foreach ($synonymData as $synonym => $terms) {
            
            if ( $synonym == "N/A" ) 
                continue;

            if (count($terms) <= 1 && ($terms[0] === $synonym || $terms[0] == "N/A")) {
                continue;
            }

            $nlu['nlu'][] = [
                'synonym' => $synonym,
                //'examples' => "| \n     - " . implode("\n     - ", $terms)
                'examples' =>"- ". implode("\n- ", $terms)
            ];
        }

        $intents = \DB::table('intentions')->where('chatbot_id', $bot)->whereNull("deleted_at")->get();


        foreach ($intents as $intent) {
            $questions = \DB::table('question')
            ->where('intentions_id', $intent->id)
            ->pluck('id');

            $trainingPhrases = \DB::table('question_languages')
            ->whereIn('question_id', $questions)
            ->where('language', $language)
            ->pluck('question');

$trainingPhrases = $trainingPhrases->toArray();
            foreach ($trainingPhrases as $phrase) {
                if (preg_match_all('/\[(.*?)\]\((.*?)\)/', $phrase, $matches)) {
                    foreach ($matches[1] as $index => $match) {
                        if (isset($synonymData[$match])) {
                            foreach ($synonymData[$match] as $synonym) {
                                $newPhrase = str_replace("[".$match."]", "[".$synonym."]", $phrase);
                                if (!in_array($newPhrase, $trainingPhrases)) {
                                    $trainingPhrases[] = $newPhrase;
                                }
                        }
                    }
                }
            }
          }

            if (is_array($trainingPhrases) && !empty($trainingPhrases)) {
                $nlu['nlu'][] = [
                    'intent' => strtolower(str_replace(" ","_",$intent->name)),
                    'examples' => "- " . implode("\n- ",$trainingPhrases)
                ];

            }
        }

        return Yaml::dump($nlu, 6, 2, Yaml::DUMP_MULTI_LINE_LITERAL_BLOCK);
   }


    public function getDefaultRulesYaml() {
        $rules = [
            "rules" => []
        ];

        $rules['rules'][] = [
            'rule' => "NLU Fallback",
            'steps' => [
                ['intent' => 'nlu_fallback'],
                ['action' => 'utter_no_le_he_entendido']
            ]
        ];
        return $rules;
    }
    public function createRulesYaml($bot, $language) {
        // TODO: Queda  crear reglas para los no formularios

        $rules = $this->getDefaultRulesYaml();

        $intentsWithConcepts = \DB::table('intentions')
            ->join('intentions_concepts', 'intentions.id', '=', 'intentions_concepts.intention_id')
            ->where('intentions.chatbot_id', $bot)->whereNull("intentions.deleted_at")
            ->select('intentions.name')
            ->distinct()
            ->pluck('name');

     


        foreach ($intentsWithConcepts as $name) {
            $form = strtolower(str_replace(" ", "_", $name));

            // Regla para activar el formulario
            $rules['rules'][] = [
                'rule' => "Activar formulario $form",
                'condition' => [
                    ['active_loop' => null]],
                'steps' => [
                        ['intent' => $form],
                        ['action' => 'action_limpiar_formulario'],
                        ['action' => "{$form}_form"],
                        ['active_loop' => "{$form}_form"]]
            ];

            //// Regla para enviar el formulario
            $rules['rules'][] = [
                'rule' => "Enviar $form",
                'condition' => [
                    ['active_loop' => "{$form}_form"]],
                'steps' => [
                    ['action' => "{$form}_form"],
                    ['active_loop' => null],
                    ['action' => 'action_terminar_formulario']]#,
                  # ['action' => 'action_limpiar_formulario']]
            ];

            $rules['rules'][] = [
                'rule' => "Cancelar $form",
                'condition' => [
                    ['active_loop' => "{$form}_form"]],
                'steps' => [
                    ['intent' => 'cancelar'],
                    ['action' => 'action_deactivate_loop'],
                    ['active_loop' => null],
                    ['action' => 'utter_cancelar'],
                    ['action' => 'action_limpiar_formulario']]
            ];

            $rules['rules'][] = [
                'rule' => "NLU Fallback $form",
                'condition' => [
                    ['active_loop' => "{$form}_form"]],
                'steps' => [
                    ['intent' => 'nlu_fallback'],
                    ['action' => 'utter_no_le_he_entendido']]
            ];
            //Regla ignorar intenciones en el formulario
            /* $intentionsToIgnoreCollection = \DB::table('intentions')
            ->where('intentions.chatbot_id', $bot)
            ->whereNotIn('intentions.name', [$form, 'cancelar', 'desvio_agente'])
            ->pluck('name');
            if($intentionsToIgnoreCollection && $intentionsToIgnoreCollection->isNotEmpty()){
                $intentionsToIgnore =  $intentionsToIgnoreCollection->toArray();
                $intentionsToIgnoreString = implode("', '", $intentionsToIgnore);
                $intentionsToIgnoreString = "'$intentionsToIgnoreString'";
                $rules['rules'][] = [
                    'rule' => "Ignorar intenciones formulario $form",
                    'condition' => [
                        ['active_loop' => "{$form}_form"]],
                    'steps' => [
                        ['intent' => $intentionsToIgnoreString],
                        ['action' => 'action_listen']]
                ];
            } */
            

        }

        $intentsWithoutConcepts = \DB::table('intentions')
                    ->leftJoin('intentions_concepts', 'intentions.id', '=', 'intentions_concepts.intention_id')
                    ->where('intentions.chatbot_id', $bot)
                    ->whereNull('intentions_concepts.intention_id')->whereNull("intentions.deleted_at")
                    ->select('intentions.name')
                    ->distinct()
                    ->pluck('name');

        foreach ( $intentsWithoutConcepts as $intentName) {
            $intent = strtolower(str_replace(" ", "_", $intentName));
                // Si $intent != desvio_agente
                // si $intent == desvio_agente, no se añade el "condition"
            if ($intent == "desvio_agente") {
                $rules['rules'][] = [
                    'rule' => "Contestar $intent",
                    'condition' => [
                        ['active_loop' => null]],
                    'steps' => [
                        ['intent' => "$intentName"],
                        ['action' => 'utter_'.$intentName]]
                ];
            } else {
                $rules['rules'][] = [
                    'rule' => "Contestar $intent",
                    'condition' => [
                        ['active_loop' => null]],
                    'steps' => [
                        ['intent' => "$intentName"],
                        ['action' => 'utter_'.$intentName]
                    ]
                ];
            }
        }

        return Yaml::dump($rules, 6, 2, Yaml::DUMP_MULTI_LINE_LITERAL_BLOCK);
    }


    public function getDefaultPipelineYaml() {
        $pipeline = [
            "assistant_id" => "default",
            "language" => "es",
            "pipeline" => [
                ["name" => "accent_normalizer.AccentNormalizer"],
                ["name" => "SpacyNLP",
                    "model" => "es_core_news_md"],
                ["name" => "WhitespaceTokenizer"],
                ["name" => "RegexFeaturizer"],
                ["name" => "RegexEntityExtractor"],
                ["name" => "LexicalSyntacticFeaturizer"],
                //["name" => "CRFEntityExtractor"],
                ["name" => "CountVectorsFeaturizer"],
                ["name" => "CountVectorsFeaturizer",
                    "analyzer" => "char_wb",
                    "min_ngram" => 1,
                    "max_ngram" => 4],
                ["name" => "DIETClassifier",
                    "epochs" => 100],
                ["name" => "EntitySynonymMapper"],
                ["name" => "ResponseSelector",
                    "epochs" => 100],
                ["name" => "FallbackClassifier",
                    "threshold" => 0.75]
            ]
        ];

        return $pipeline;
    }


    public function createPipelineYaml($bot, $language) {
        $pipeline = $this->getDefaultPipelineYaml();
        $id_exito = \DB::table("default_settings")->select("id")->where("name","porcentaje_exito")->first()->id;
        $thres = \DB::table("chatbot_settings")->select("value")->whereNull("deleted_at")->where("chatbot_id", $bot)->where("default_id", $id_exito)->first()->value;
        // Modify  the "threshold" element from the last element of "pipeline" in $pipeline variable with $thres value
        $pipeline["pipeline"][count($pipeline["pipeline"])-1]["threshold"] = $thres/100.0;


        switch ($language) {
            case 'castellano':
                $pipeline['pipeline'][0]['model'] = 'es_core_news_md';
                break;
            case 'catalan':
                $pipeline['pipeline'][0]['model'] = 'ca_core_news_md';
                break;
            case 'ingles':
                $pipeline['pipeline'][0]['model'] = 'en_core_web_md';
                break;
        }

        return Yaml::dump($pipeline, 6, 2, Yaml::DUMP_MULTI_LINE_LITERAL_BLOCK);
    }

    public function getDefaultRulesPolicyYaml() {
        $policies = ["policies" => [
            //["name" => "MemoizationPolicy"],
            //["name" => "TEDPolicy",
            //    "max_history" => 5,
            //    "epochs" => 100],
            ["name" => "RulePolicy"]
        ]];

        return $policies;
    }
    public function createRulesPolicyYaml($bot, $language) {
        $policies = $this->getDefaultRulesPolicyYaml();

        return Yaml::dump($policies, 6, 2, Yaml::DUMP_MULTI_LINE_LITERAL_BLOCK);

    }

    public function getDefaultStoriesYaml() {
        $stories = ["stories" => [[]]];

        return $stories;
    }
    public function createStoriesYaml($bot, $language) {

        $stories = $this->getDefaultStoriesYaml();

        return Yaml::dump($stories, 6, 2, Yaml::DUMP_MULTI_LINE_LITERAL_BLOCK);
    }
}
